(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/core')) :
	typeof define === 'function' && define.amd ? define(['exports', '@angular/core'], factory) :
	(factory((global.ngx = global.ngx || {}, global.ngx.clipboard = global.ngx.clipboard || {}),global.ng.core));
}(this, (function (exports,_angular_core) { 'use strict';

var ClipboardService = (function () {
    /**
     * @param {?} renderer
     */
    function ClipboardService(renderer) {
        this.renderer = renderer;
    }
    Object.defineProperty(ClipboardService.prototype, "isSupported", {
        /**
         * @return {?}
         */
        get: function () {
            return !!document.queryCommandSupported && !!document.queryCommandSupported('copy');
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @param {?} element
     * @return {?}
     */
    ClipboardService.prototype.isTargetValid = function (element) {
        if (element && typeof element === 'object' && element.nodeType === 1) {
            if (element.hasAttribute('disabled')) {
                throw new Error('Invalid "target" attribute. Please use "readonly" instead of "disabled" attribute');
            }
            return true;
        }
        throw new Error('Invalid "target" value, use a valid Element');
    };
    /**
     * copyFromInputElement
     * @param {?} targetElm
     * @return {?}
     */
    ClipboardService.prototype.copyFromInputElement = function (targetElm) {
        this.selectTarget(targetElm);
        var /** @type {?} */ re = this.copyText();
        this.clearSelection(targetElm, window);
        return re;
    };
    /**
     * Creates a fake textarea element, sets its value from `text` property,
     * and makes a selection on it.
     * @param {?} content
     * @return {?}
     */
    ClipboardService.prototype.copyFromContent = function (content) {
        if (this.tempTextArea) {
            this.tempTextArea = this.createTempTextArea(document, window);
            document.body.appendChild(this.tempTextArea);
        }
        this.tempTextArea.value = content;
        return this.copyFromInputElement(this.tempTextArea);
    };
    /**
     * @return {?}
     */
    ClipboardService.prototype.destroy = function () {
        if (this.tempTextArea) {
            document.body.removeChild(this.tempTextArea);
            this.tempTextArea = undefined;
        }
    };
    /**
     * @param {?} inputElement
     * @return {?}
     */
    ClipboardService.prototype.selectTarget = function (inputElement) {
        // TODO: handle contenteditable or selection
        if (inputElement.nodeName === 'INPUT' || inputElement.nodeName === 'TEXTAREA') {
            // TODO: might need to set readonly first
            this.renderer.invokeElementMethod(inputElement, 'select');
            this.renderer.invokeElementMethod(inputElement, 'setSelectionRange', [0, inputElement.value.length]);
        }
    };
    /**
     * @return {?}
     */
    ClipboardService.prototype.copyText = function () {
        try {
            // TODO: shouldn't use document directly
            return document.execCommand('copy');
        }
        catch (err) {
            throw new Error('Fail to perform copy');
        }
    };
    /**
     * @param {?} inputElement
     * @param {?} window
     * @return {?}
     */
    ClipboardService.prototype.clearSelection = function (inputElement, window) {
        inputElement && inputElement.blur();
        // TODO: should inject window
        window.getSelection().removeAllRanges();
    };
    /**
     * @param {?} doc
     * @param {?} window
     * @return {?}
     */
    ClipboardService.prototype.createTempTextArea = function (doc, window) {
        var /** @type {?} */ isRTL = doc.documentElement.getAttribute('dir') === 'rtl';
        var /** @type {?} */ ta;
        ta = doc.createElement('textarea');
        // Prevent zooming on iOS
        ta.style.fontSize = '12pt';
        // Reset box model
        ta.style.border = '0';
        ta.style.padding = '0';
        ta.style.margin = '0';
        // Move element out of screen horizontally
        ta.style.position = 'absolute';
        ta.style[isRTL ? 'right' : 'left'] = '-9999px';
        // Move element to the same position vertically
        var /** @type {?} */ yPosition = window.pageYOffset || doc.documentElement.scrollTop;
        ta.style.top = yPosition + 'px';
        ta.setAttribute('readonly', '');
        return ta;
    };
    return ClipboardService;
}());
ClipboardService.decorators = [
    { type: _angular_core.Injectable },
];
/**
 * @nocollapse
 */
ClipboardService.ctorParameters = function () { return [
    { type: _angular_core.Renderer, },
]; };

var ClipboardDirective = (function () {
    /**
     * @param {?} clipboardSrv
     */
    function ClipboardDirective(clipboardSrv) {
        this.clipboardSrv = clipboardSrv;
        this.cbOnSuccess = new _angular_core.EventEmitter();
        this.cbOnError = new _angular_core.EventEmitter();
    }
    /**
     * @return {?}
     */
    ClipboardDirective.prototype.ngOnInit = function () {
        if (!this.clipboardSrv.isSupported) {
            throw new Error('Your environment does not support copy.');
        }
    };
    /**
     * @return {?}
     */
    ClipboardDirective.prototype.ngOnDestroy = function () {
        this.clipboardSrv.destroy();
    };
    /**
     * @param {?} button
     * @return {?}
     */
    ClipboardDirective.prototype.onClick = function (button) {
        if (this.targetElm && this.clipboardSrv.isTargetValid(this.targetElm)) {
            this.handleResult(this.clipboardSrv.copyFromInputElement(this.targetElm), this.targetElm.value);
        }
        else if (this.cbContent) {
            this.handleResult(this.clipboardSrv.copyFromContent(this.cbContent), this.cbContent);
        }
        else {
            throw new Error('Expecting a HtmlInputElement for ngxClipboard or a string value for cbContent.');
        }
    };
    /**
     * Fires an event based on the copy operation result.
    \@param {Boolean} succeeded
     * @param {?} succeeded
     * @param {?} copiedContent
     * @return {?}
     */
    ClipboardDirective.prototype.handleResult = function (succeeded, copiedContent) {
        if (succeeded) {
            this.cbOnSuccess.emit({ isSuccess: true, content: copiedContent });
        }
        else {
            this.cbOnError.emit({ isSuccess: false });
        }
    };
    return ClipboardDirective;
}());
ClipboardDirective.decorators = [
    { type: _angular_core.Directive, args: [{
                selector: '[ngxClipboard]'
            },] },
];
/**
 * @nocollapse
 */
ClipboardDirective.ctorParameters = function () { return [
    { type: ClipboardService, },
]; };
ClipboardDirective.propDecorators = {
    'targetElm': [{ type: _angular_core.Input, args: ['ngxClipboard',] },],
    'cbContent': [{ type: _angular_core.Input },],
    'cbOnSuccess': [{ type: _angular_core.Output },],
    'cbOnError': [{ type: _angular_core.Output },],
    'onClick': [{ type: _angular_core.HostListener, args: ['click', ['$event.target'],] },],
};

var ClipboardModule = (function () {
    function ClipboardModule() {
    }
    return ClipboardModule;
}());
//static forRoot(): ModuleWithProviders { return { ngModule: ClipboardModule, providers: [] }; }
ClipboardModule.decorators = [
    { type: _angular_core.NgModule, args: [{
                declarations: [ClipboardDirective],
                exports: [ClipboardDirective],
                providers: [ClipboardService]
            },] },
];
/**
 * @nocollapse
 */
ClipboardModule.ctorParameters = function () { return []; };

exports.ClipboardModule = ClipboardModule;
exports.ClipboardDirective = ClipboardDirective;

Object.defineProperty(exports, '__esModule', { value: true });

})));
