import { Injectable, Renderer } from '@angular/core';
var ClipboardService = (function () {
    /**
     * @param {?} renderer
     */
    function ClipboardService(renderer) {
        this.renderer = renderer;
    }
    Object.defineProperty(ClipboardService.prototype, "isSupported", {
        /**
         * @return {?}
         */
        get: function () {
            return !!document.queryCommandSupported && !!document.queryCommandSupported('copy');
        },
        enumerable: true,
        configurable: true
    });
    /**
     * @param {?} element
     * @return {?}
     */
    ClipboardService.prototype.isTargetValid = function (element) {
        if (element && typeof element === 'object' && element.nodeType === 1) {
            if (element.hasAttribute('disabled')) {
                throw new Error('Invalid "target" attribute. Please use "readonly" instead of "disabled" attribute');
            }
            return true;
        }
        throw new Error('Invalid "target" value, use a valid Element');
    };
    /**
     * copyFromInputElement
     * @param {?} targetElm
     * @return {?}
     */
    ClipboardService.prototype.copyFromInputElement = function (targetElm) {
        this.selectTarget(targetElm);
        var /** @type {?} */ re = this.copyText();
        this.clearSelection(targetElm, window);
        return re;
    };
    /**
     * Creates a fake textarea element, sets its value from `text` property,
     * and makes a selection on it.
     * @param {?} content
     * @return {?}
     */
    ClipboardService.prototype.copyFromContent = function (content) {
        if (this.tempTextArea) {
            this.tempTextArea = this.createTempTextArea(document, window);
            document.body.appendChild(this.tempTextArea);
        }
        this.tempTextArea.value = content;
        return this.copyFromInputElement(this.tempTextArea);
    };
    /**
     * @return {?}
     */
    ClipboardService.prototype.destroy = function () {
        if (this.tempTextArea) {
            document.body.removeChild(this.tempTextArea);
            this.tempTextArea = undefined;
        }
    };
    /**
     * @param {?} inputElement
     * @return {?}
     */
    ClipboardService.prototype.selectTarget = function (inputElement) {
        // TODO: handle contenteditable or selection
        if (inputElement.nodeName === 'INPUT' || inputElement.nodeName === 'TEXTAREA') {
            // TODO: might need to set readonly first
            this.renderer.invokeElementMethod(inputElement, 'select');
            this.renderer.invokeElementMethod(inputElement, 'setSelectionRange', [0, inputElement.value.length]);
        }
    };
    /**
     * @return {?}
     */
    ClipboardService.prototype.copyText = function () {
        try {
            // TODO: shouldn't use document directly
            return document.execCommand('copy');
        }
        catch (err) {
            throw new Error('Fail to perform copy');
        }
    };
    /**
     * @param {?} inputElement
     * @param {?} window
     * @return {?}
     */
    ClipboardService.prototype.clearSelection = function (inputElement, window) {
        inputElement && inputElement.blur();
        // TODO: should inject window
        window.getSelection().removeAllRanges();
    };
    /**
     * @param {?} doc
     * @param {?} window
     * @return {?}
     */
    ClipboardService.prototype.createTempTextArea = function (doc, window) {
        var /** @type {?} */ isRTL = doc.documentElement.getAttribute('dir') === 'rtl';
        var /** @type {?} */ ta;
        ta = doc.createElement('textarea');
        // Prevent zooming on iOS
        ta.style.fontSize = '12pt';
        // Reset box model
        ta.style.border = '0';
        ta.style.padding = '0';
        ta.style.margin = '0';
        // Move element out of screen horizontally
        ta.style.position = 'absolute';
        ta.style[isRTL ? 'right' : 'left'] = '-9999px';
        // Move element to the same position vertically
        var /** @type {?} */ yPosition = window.pageYOffset || doc.documentElement.scrollTop;
        ta.style.top = yPosition + 'px';
        ta.setAttribute('readonly', '');
        return ta;
    };
    return ClipboardService;
}());
export { ClipboardService };
ClipboardService.decorators = [
    { type: Injectable },
];
/**
 * @nocollapse
 */
ClipboardService.ctorParameters = function () { return [
    { type: Renderer, },
]; };
function ClipboardService_tsickle_Closure_declarations() {
    /** @type {?} */
    ClipboardService.decorators;
    /**
     * @nocollapse
     * @type {?}
     */
    ClipboardService.ctorParameters;
    /** @type {?} */
    ClipboardService.prototype.tempTextArea;
    /** @type {?} */
    ClipboardService.prototype.renderer;
}
//# sourceMappingURL=clipboard.service.js.map